# Generated by Django 2.0.1 on 2018-01-26 10:20

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('billing', '0009_longer_product_code'),
    ]

    operations = [
        migrations.AlterField(
            model_name='productproperty',
            name='value',
            field=models.CharField(blank=True, max_length=255),
        ),
    ]
