# -*- coding: utf-8 -*-
# Generated by Django 1.11.6 on 2017-12-19 17:37
from __future__ import unicode_literals

from django.db import migrations
import django_fsm


class Migration(migrations.Migration):

    dependencies = [
        ('billing', '0004_add_modified_field'),
    ]

    operations = [
        migrations.AddField(
            model_name='creditcard',
            name='status',
            field=django_fsm.FSMField(choices=[('ACTIVE', 'Active'), ('INACTIVE', 'Inactive')], db_index=True, default='ACTIVE', max_length=20),
        ),
    ]
