# Generated by Django 2.0.2 on 2018-02-10 13:26

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        ('billing', '0013_replace_past_due_status_by_due_date'),
    ]

    operations = [
        migrations.AlterField(
            model_name='charge',
            name='invoice',
            field=models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.SET_NULL, related_name='items', to='billing.Invoice'),
        ),
    ]
