# -*- coding: utf-8 -*-
# Generated by Django 1.11.6 on 2017-11-10 08:46
from __future__ import unicode_literals

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):
    dependencies = [
        ('contenttypes', '0002_remove_content_type_name'),
        ('billing', '0002_default_currency'),
    ]

    operations = [
        migrations.RemoveField(
            model_name='creditcard',
            name='psp_uri',
        ),
        migrations.RemoveField(
            model_name='transaction',
            name='psp_uri',
        ),
        migrations.AddField(
            model_name='creditcard',
            name='psp_content_type',
            field=models.ForeignKey(default=1, on_delete=django.db.models.deletion.CASCADE,
                                    to='contenttypes.ContentType'),
            preserve_default=False,
        ),
        migrations.AddField(
            model_name='creditcard',
            name='psp_object_id',
            field=models.UUIDField(db_index=True, default='00000000-0000-0000-0000-000000000000'),
            preserve_default=False,
        ),
        migrations.AddField(
            model_name='transaction',
            name='psp_content_type',
            field=models.ForeignKey(default=1, on_delete=django.db.models.deletion.CASCADE,
                                    to='contenttypes.ContentType'),
            preserve_default=False,
        ),
        migrations.AddField(
            model_name='transaction',
            name='psp_object_id',
            field=models.UUIDField(db_index=True, default='00000000-0000-0000-0000-000000000000'),
            preserve_default=False,
        ),
    ]
