# Generated by Django 2.1.4 on 2018-12-23 15:07

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        ('billing', '0014_deleting_invoice_detaches_charges'),
    ]

    operations = [
        migrations.AlterField(
            model_name='transaction',
            name='psp_content_type',
            field=models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.CASCADE, to='contenttypes.ContentType'),
        ),
        migrations.AlterField(
            model_name='transaction',
            name='psp_object_id',
            field=models.UUIDField(blank=True, db_index=True, null=True),
        ),
    ]
