# Generated by Django 2.0.1 on 2018-01-21 13:34

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        ('billing', '0007_payed_to_paid'),
    ]

    operations = [
        migrations.AddField(
            model_name='charge',
            name='deleted',
            field=models.BooleanField(default=False),
        ),
        migrations.AddField(
            model_name='charge',
            name='reverses',
            field=models.OneToOneField(blank=True, null=True, on_delete=django.db.models.deletion.PROTECT, related_name='reversed_by', to='billing.Charge'),
        ),
    ]
