# Generated by Django 2.0.1 on 2018-01-26 09:41

import django.core.validators
from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('billing', '0008_charge_deleted_and_reverses'),
    ]

    operations = [
        migrations.AlterField(
            model_name='charge',
            name='product_code',
            field=models.CharField(blank=True, db_index=True, help_text='Identifies the kind of product being charged or credited', max_length=10, validators=[django.core.validators.RegexValidator(message='Between 4 and 10 uppercase letters or digits', regex='^[A-Z0-9]{4,10}$')]),
        ),
    ]
